import { describe, it, expect, beforeAll } from "@jest/globals";
import AeternityService from "../src/services/AeternityService";
import { RequestError } from "../src/utils/requestAdapter";

describe("AeternityService tests", () => {
  describe("Tests requestApi() function", () => {
    let aeternityService: AeternityService;

    beforeAll(() => {
      const NOT_EXISTING_ADDRESS = "https://random123abcdef.com";
      const MALFUNCTIONING_NODE = "https://malfunctioning-ae-node.com";
      const WORKING_NODE = "https://working-ae-node.com";
      const LAST_WORKING_NODE = "https://last-working-ae-node.com";

      aeternityService = new AeternityService(
        [
          NOT_EXISTING_ADDRESS,
          MALFUNCTIONING_NODE,
          WORKING_NODE,
          LAST_WORKING_NODE,
        ],
        [""],
        "contract bytecode"
      );

      // Mock request
      aeternityService.requestWrapper = async (config) => {
        const { url } = config;

        if (url.includes(WORKING_NODE)) {
          if (url.includes("trigger_404")) {
            throw new RequestError("404 Not found", config, {
              data: "404 Not found",
              status: 404,
              statusText: "404 Not found",
              headers: {},
              config,
            });
          } else if (url.includes("invalid_input")) {
            throw new RequestError("Invalid input", config, {
              data: "Invalid input",
              status: 400,
              statusText: "Invalid input",
              headers: {},
              config,
            });
          }

          return {
            data: "success",
            status: 200,
            statusText: "OK",
            headers: {},
            config,
          };
        } else if (url.includes(MALFUNCTIONING_NODE)) {
          throw new RequestError("Error", config, {
            data: "Error",
            status: 403,
            statusText: "Error",
            headers: {},
            config,
          });
        }

        throw new RequestError("Not found", config);
      };
    });

    it("Checks chain of successful retry attempts", async () => {
      const response = await aeternityService.requestNode({
        url: "/test/node/api",
      });
      expect(response.data).toEqual("success");
    });

    it("Checks 404", async () => {
      try {
        await aeternityService.requestNode({
          url: "/trigger_404",
        });
      } catch (e) {
        expect(e.response.data).toEqual("404 Not found");
      }
    });

    it("Checks invalid input", async () => {
      try {
        await aeternityService.requestNode({
          url: "/invalid_input",
        });
      } catch (e) {
        expect(e.response.data).toEqual("Invalid input");
      }
    });
  });
});
