import { describe, it, expect } from "@jest/globals";
import { CryptoService, getMerkleTreeRootHash } from "../src";

describe("Merkle Tree root hash test", () => {
  const rootHashesAndNodes = {
    "ypeBEsobvcr6wjGzmiPcTaeG7/gUfE5yuYB3ha/uSLs=": ["a"],
    "5aAf7hTg7VxIcU8iGA8lrYNltT+XefedxKPX6Tlj+Uo=": ["a", "b"],
    "cHUVLQOlzZIQSIe0doYneOwMh75cL6HAqQ+HxJ+tbv8=": ["a", "b", "c"],
    "FO3l6Ol62TcjJ3KPUJm5VgSjlZPKw704o0OtdiBSE+c=": ["a", "b", "c", "d"],
    "1x+Jg61O4XD4Ep8evN10QL53mNjhyAQgvxHx7O1hDbo=": ["a", "b", "c", "d", "e"],
    "H3N5U5cHvK6gBWQWjR1NYmsJtz+KKjZSNMYtdj+FTaI=": [
      "a",
      "b",
      "c",
      "d",
      "e",
      "f",
    ],
  };

  it("Should build a proper root hash", async () => {
    await Promise.all(
      Object.keys(rootHashesAndNodes).map(async (rootHash) => {
        const nodes = rootHashesAndNodes[rootHash];

        const leaves = (await Promise.all(
          nodes.map(async (x) => Buffer.from(await CryptoService.SHA256(x)))
        )) as Buffer[];

        const hash = await getMerkleTreeRootHash(leaves);
        expect(Buffer.from(hash).toString("base64")).toEqual(rootHash);
      })
    );
  });
});
